f64 StringToFloat64(string value) {
    b32 is_negative = value.base[0] == '-';

    f64 result = 0.0;
    u32 digits = 0;
    for (u32 i = is_negative; i < value.length; i++) {
        u8 potential_digit = (value.base[i] - '0');
        b32 is_digit = potential_digit < 10;
        if (is_digit) {
            digits = (digits * 10) + potential_digit;
        } else if (value.base[i] == '.') {
            i += 1; // We know we have just 1 digit after the decimal point so read just that
            u8 fraction = (value.base[i] - '0');
            result = ((f64)digits) + (0.1 * fraction);
            break; // Break out of the loop
        }
    }

    return (result);
}

void RunAttempt01(memory_arena *arena, program_times *times, attempt_type type) {
    StartProfilerZonePtr(&times->overall);

    hash_map map = HashMap(arena, HASH_MAP_DEFAULT_CAPACITY);

    StartProfilerZonePtr(&times->reading);
    file_map file = OpenFileMap(g_measurements_filename);

    u64 start = file.buffer.position;
    for (;file.buffer.position < file.buffer.capacity; file.buffer.position++) {
        if (file.buffer.base[file.buffer.position] == ';') {
            string name = String(&file.buffer.base[start], (u32)(file.buffer.position - start));

            file.buffer.position += 1;
            start = file.buffer.position;
            for (; file.buffer.position < file.buffer.capacity; file.buffer.position++) {
                if (file.buffer.base[file.buffer.position] == '\n') {
                    string temp_str = String(&file.buffer.base[start], (u32)(file.buffer.position - start));
                    f64 temperature = StringToFloat64(temp_str);

                    hash_map_bucket *bucket = HashMapInsertValue(&map, name, temperature);
                    Unused(bucket);

                    start = file.buffer.position + 1;
                    break; // continue with main loop
                }
            }
        }
    }
    EndProfilerZone(&times->reading);

    StartProfilerZonePtr(&times->output);
    if (type != ATTEMPT_TYPE_BENCHMARK) {
        WriteMessage(StringLiteral("{"));
    }

    b32 is_first = B32_TRUE;
    for (u32 i = 0; i < map.capacity; i++) {
        if (map.base[i].key.length != 0) {
            s8 *format;
            if (is_first) {
                is_first = B32_FALSE;
                format = "%.*s=%.1f/%.1f/%.1f";
            } else {
                format = ", %.*s=%.1f/%.1f/%.1f";
            }

            string name = map.base[i].key;
            f64 average = map.base[i].average / (f64)map.base[i].count;
            f64 min = map.base[i].min;
            f64 max = map.base[i].max;

            u64 arena_pos = GetMemoryArenaPosition(arena);
            string message = StringFormat(arena, format, StringToFormatArgs(name), min, average, max);

            if (type != ATTEMPT_TYPE_BENCHMARK) {
                WriteMessage(message);
            }

            RewindMemoryArenaPosition(arena, arena_pos);
        }
    }

    if (type != ATTEMPT_TYPE_BENCHMARK) {
        WriteMessage(StringLiteral("}\n"));
    }
    EndProfilerZone(&times->output);

    CloseFileMap(&file);

    EndProfilerZone(&times->overall);
}
