typedef struct ARGUMENT_CONTEXT argument_context;
struct ARGUMENT_CONTEXT {
    u8 *base;
    u32 length;
    u32 position;
};

static argument_context g_argument_context = {0};

argument_context * GetArgumentContext(void) {
    argument_context *result = &g_argument_context;
    return (result);
}

#define ArgumentContext(name) argument_context *name = GetArgumentContext()

void ArgumentSkipUntilQuote(void) {
    while (g_argument_context.position < g_argument_context.length) {
        u8 character = g_argument_context.base[g_argument_context.position];
        g_argument_context.position++;

        switch (character) {
            case '"': return;
            case '\\': {
                if (g_argument_context.position < g_argument_context.length) {
                    g_argument_context.position++;
                }
            }
        }
    }
}

void ArgumentSkipSeparators(void) {
    while (g_argument_context.position < g_argument_context.length) {
        u8 character = g_argument_context.base[g_argument_context.position];

        switch (character) {
            case ' ':
            case '\t': {
                g_argument_context.position++;
                break;
            }
            default: return;
        }
    }
}

void ArgumentSkipUntilSeparator(void) {
    while (g_argument_context.position < g_argument_context.length) {
        u8 character = g_argument_context.base[g_argument_context.position];

        if (character == ' ' || character == '\t') {
            return;
        }

        g_argument_context.position++;

        if (character == '"') {
            ArgumentSkipUntilQuote();
        }
    }
}

string GetNextCommandLineArgument(void) {
    string result = {0};

    if (g_argument_context.base == 0) {
        g_argument_context.base = (u8 *)GetCommandLineA();
        g_argument_context.length = (u32)strlen((s8 *)g_argument_context.base); // ok to cast down since windows has a cmd limit of 8191 chars
    }

    if (g_argument_context.position < g_argument_context.length) {
        ArgumentSkipSeparators();

        u32 start = g_argument_context.position;

        ArgumentSkipUntilSeparator();

        u32 end = g_argument_context.position;

        result.base = &g_argument_context.base[start];
        result.length = (end - start);

        if (result.length == 0) {
            result.base = 0;
        }
    }

    return (result);
}
