#define CONSOLE_MESSAGE_BUFFER_LENGTH 1024

typedef struct CONSOLE_CONTEXT console_context;
struct CONSOLE_CONTEXT {
    void *input;
    void *output;
    void *error;

    s8 message_buffer[CONSOLE_MESSAGE_BUFFER_LENGTH];
};

static console_context g_console_context = {0};

#define EXIT_CODE_NO_CONSOLE (0x20000000 + 1)

string GetLastWindowsErrorMessage(u32 error_code) {
    string result = {0};
    u32 format_flags = FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS;
    result.length = FormatMessageA(format_flags, 0, error_code, 0, (LPSTR)&StringToCString(result), 0, 0);
    return (result);
}

void FatalWindowsError(u32 error_code) {
    string message = GetLastWindowsErrorMessage(error_code);

    u32 length = 0;
    if (message.length == 0) {
        length = stbsp_snprintf(g_console_context.message_buffer, CONSOLE_MESSAGE_BUFFER_LENGTH,
            "Program terminated due to a fatal error. A detailed error message could not be retrieved from the system. The error code is %lu.",
            error_code);
    } else {
        length = stbsp_snprintf(g_console_context.message_buffer, CONSOLE_MESSAGE_BUFFER_LENGTH,
            "Program terminated due to a fatal error: %.*s",
            StringToFormatArgs(message));
    }

    WriteConsoleA(g_console_context.error, g_console_context.message_buffer, length, 0, 0);
    ExitProcess(error_code); // TODO: maybe use TerminateProcess instead?
}

void SetUpConsole(void) {
    g_console_context.input = GetStdHandle(STD_INPUT_HANDLE);
    if (g_console_context.input == INVALID_HANDLE_VALUE) {
        ExitProcess(EXIT_CODE_NO_CONSOLE);
    }

    g_console_context.output = GetStdHandle(STD_OUTPUT_HANDLE);
    if (g_console_context.output == INVALID_HANDLE_VALUE) {
        ExitProcess(EXIT_CODE_NO_CONSOLE);
    }

    g_console_context.error = GetStdHandle(STD_ERROR_HANDLE);
    if (g_console_context.error == INVALID_HANDLE_VALUE) {
        ExitProcess(EXIT_CODE_NO_CONSOLE);
    }

    if (SetConsoleCP(CP_UTF8) == 0) {
        FatalWindowsError(GetLastError());
    }

    if (SetConsoleOutputCP(CP_UTF8) == 0) {
        FatalWindowsError(GetLastError());
    }

    SetLastError(ERROR_SUCCESS);
}

#define WriteMessage(message) WriteConsoleA(g_console_context.output, StringToCString((message)), (message).length, 0, 0)
#define WriteError(message) WriteConsoleA(g_console_context.output, StringToCString((message)), (message).length, 0, 0)
