typedef struct FILE_BUFFER file_buffer;
struct FILE_BUFFER {
    u8 *base;
    u64 capacity;
    u64 position;
};

typedef struct FILE_MAP file_map;
struct FILE_MAP {
    void *handle;
    void *mapping;
    file_buffer buffer;
};

file_map OpenFileMap(string filename) {
    void *handle = CreateFile(StringToCString(filename), GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0);
    if (handle == INVALID_HANDLE_VALUE) {
        FatalWindowsError(GetLastError());
    }

    LARGE_INTEGER file_size;
    if (GetFileSizeEx(handle, &file_size) == 0) {
       FatalWindowsError(GetLastError());
    }

    void *mapping = CreateFileMapping(handle, 0, PAGE_READONLY, 0, 0, 0);
    if (mapping == INVALID_HANDLE_VALUE || mapping == 0) {
        FatalWindowsError(GetLastError());
    }

    u8 *base = (u8 *)MapViewOfFile(mapping, FILE_MAP_READ, 0, 0, file_size.QuadPart);
    if (base == 0) {
        FatalWindowsError(GetLastError());
    }

    file_map result;
    result.handle = handle;
    result.mapping = mapping;
    result.buffer.base = base;
    result.buffer.capacity = file_size.QuadPart;
    result.buffer.position = 0;
    return (result);
}

void CloseFileMap(file_map *file) {
    UnmapViewOfFile(file->buffer.base);
    CloseHandle(file->mapping);
    CloseHandle(file->handle);

    memset(file, 0, sizeof(*file));
}

file_buffer FileBuffer(memory_arena *arena, u64 capacity) {
    file_buffer result = {0};

    result.capacity = capacity;
    result.base = GetMemoryArenaChunk(arena, capacity);

    return (result);
}

void * CreateFileToWrite(string filename) {
    void *result = 0;
    result = CreateFileA(StringToCString(filename), GENERIC_WRITE, 0, 0, CREATE_ALWAYS, 0, 0);
    if (result == INVALID_HANDLE_VALUE) {
        FatalWindowsError(GetLastError());
    }
    return (result);
}

void WriteFileBufferToFile(void *file, file_buffer *buffer) {
    if (file) {
        WriteFile(file, &buffer->base[0], (DWORD)buffer->position, 0, 0);
    }
    buffer->position = 0;
}

void CloseFile(void *file) {
    if (file) {
        CloseHandle(file);
    }
}

void WriteStringToFileBuffer(void *file, file_buffer *buffer, string value) {
    u64 remaining = buffer->capacity - buffer->position;
    if (remaining < value.length) {
        WriteFileBufferToFile(file, buffer);
    }
    memcpy(&buffer->base[buffer->position], value.base, value.length);
    buffer->position += value.length;
}
