typedef struct HASH_MAP_BUCKET hash_map_bucket;
struct HASH_MAP_BUCKET {
    string key;
    u32 hash;
    
    u64 count;
    f64 average;
    f64 min;
    f64 max;
};

typedef struct HASH_MAP hash_map;
struct HASH_MAP {
    hash_map_bucket *base;
    u32 capacity;
};

#define HASH_MAP_DEFAULT_CAPACITY 512

hash_map HashMap(memory_arena *arena, u32 capacity) {
    hash_map result = {0};
    result.capacity = capacity;
    result.base = GetMemoryArenaArray(arena, hash_map_bucket, result.capacity);
    memset(result.base, 0, sizeof(*result.base) * result.capacity);
    return result;
}

// http://www.cse.yorku.ca/~oz/hash.html
// DJB2 - Dan Bernstein
u32 ComputeHash(u8 *name, u32 length) {
    u8 *end = name + length;
    u32 hash = 5381;

    while (name != end) {
        hash = ((hash << 5) + hash) + *(name++);
    }

    return hash;
}

hash_map_bucket * InsertOrUpdateBucket(hash_map *map, u32 from, u32 to, string key, u32 hash, f64 temperature) {
    hash_map_bucket *result = 0;

    for (u32 i = from; i < to; i++) {
        hash_map_bucket *bucket = &map->base[i];

        if (bucket->key.length == 0) {
            // Slot is empty so insert
            bucket->key = key;
            bucket->hash = hash;
            bucket->count = 1;
            bucket->average = temperature;
            bucket->min = temperature;
            bucket->max = temperature;
            result = bucket;
            break; // stop the loop

        } else if (bucket->hash == hash && IsStringEqual(key, bucket->key)) {
            // Update the item
            bucket->count += 1;
            bucket->average += temperature;
            if (temperature < bucket->min) {
                bucket->min = temperature;
            }
            if (temperature > bucket->max) {
                bucket->max = temperature;
            }
            result = bucket;
            break; // stop the loop
        }

        // Not empty or already occupied by something else... continue search
    }

    return (result);
}

hash_map_bucket * HashMapInsertValue(hash_map *map, string key, f64 temperature) {
    u32 hash = ComputeHash(key.base, key.length);
    u32 slot = hash % map->capacity;

    // Insert from slot to bottom
    hash_map_bucket *result = InsertOrUpdateBucket(map, slot, map->capacity, key, hash, temperature);
    if (result == 0) {
        result = InsertOrUpdateBucket(map, 0, slot, key, hash, temperature);
    }
    return (result);
}
