u64 ReadOSTimer(void) {
    LARGE_INTEGER timer;
    QueryPerformanceCounter(&timer);
    return timer.QuadPart;
}

u64 ReadOSTimerFrequency(void) {
    LARGE_INTEGER frequency;
    QueryPerformanceFrequency(&frequency);
    return frequency.QuadPart;
}

// TODO: RDPRU / RDPMC instead on Zen2??? Because AMD has large granularity of 40 or so cycles??? Check CPU Manual!!!!!
u64 ReadCPUTimer(void) {
    __faststorefence();
    u64 result = __rdtsc();
    return (result);
}

u64 ReadFencedCPUTimer(void) {
    unsigned int aux;
    u64 result = __rdtscp(&aux);
    __faststorefence();
    return result;
}

u64 ReadCPUTimerFrequency(u64 wait_time_ms) {
    u64 os_frequency = ReadOSTimerFrequency(); // Counts per second
    u64 os_wait_time = os_frequency * wait_time_ms / 1000;

    u64 cpu_start = ReadCPUTimer();
    u64 os_start = ReadOSTimer();

    u64 os_end;
    u64 os_elapsed = 0;

    while (os_elapsed < os_wait_time) {
        os_end = ReadOSTimer();
        os_elapsed = os_end - os_start;
    }

    u64 cpu_end = ReadCPUTimer();

    u64 cpu_elapsed = cpu_end - cpu_start;
    u64 cpu_frequency = 0;

    if (os_elapsed)
        cpu_frequency = os_frequency * cpu_elapsed / os_elapsed;

    return cpu_frequency;
}

void WarmupProfiler(void) {
    ReadCPUTimer();
    ReadCPUTimer();
    ReadCPUTimer();
    ReadFencedCPUTimer();
}

typedef struct PROFILER_ZONE profiler_zone;
struct PROFILER_ZONE {
    u64 ticks_start;
    u64 ticks_end;
};

void StartProfilerZonePtr(profiler_zone *zone) {
    zone->ticks_start = ReadCPUTimer();
}

profiler_zone StartProfilerZone() {
    profiler_zone result;
    result.ticks_start = ReadCPUTimer();
    result.ticks_end = 0;
    return result;
}

void EndProfilerZone(profiler_zone *zone) {
    zone->ticks_end = ReadFencedCPUTimer();
}
